/******************************************************************************* 
*  Charger_Functions.h is a header file containing functions for ATTiny85 for:
*  
*  1. Initialization of the ADC initADC
*  2. Reading and returning smoothed 10-bit ADC value
*  3. Definition of discharge mode
*  4. Definition of stand-by, floating mode
*  5. Charge/discharge input debouncing 60ms
*  Author: George Novacek
*  Created June 10, 2013
*******************************************************************************/

#ifndef CHARGER_FUNCTIONS_H_
#define CHARGER_FUNCTIONS_H_

int batVolts;                               // battery voltage
int alpha = 7;                              // filter coefficient
uint8_t x = 0;                              // 1 discharge, 0 charge


// ADC INITIALIZATION ==========================================================
void initADC(void)
{
    ADMUX = (0<<ADLAR) |                    // right adjust result
            (0<<REFS1) |                    // reference voltage to Vcc
            (0<<REFS0) |                    // reference voltage to Vcc
            (0<<REFS2) |                    // reference voltage to Vcc
            (0<<MUX3) |                     // use ADC2 (PB4) for input
            (0<<MUX2) |                     // use ADC2 (PB4) for input
            (0<<MUX1) |                     // use ADC2 (PB4) for input
            (1<<MUX0);                      // use ADC2 (PB4) for input
            
   ADCSRA = (1<<ADEN) |                     // enable ADC
            (1<<ADPS2) |                    // set prescaler to 64
            (1<<ADPS1) |                    // set prescaler to 64
            (0<<ADPS0);                     // set prescaler to 64
}

// ReadADC =====================================================================
void readADC()
{
  static int prevRead = 0;                  // previously read battery voltage
  int lowADC;
  int highADC;
	
	ADCSRA |=(1<<ADSC);                     // start the conversion
    while (ADCSRA & (1<<ADSC))              // wait until done
        
    highADC = (int)ADCH;                    // bits 8 and 9, convert to int
	lowADC = (int)ADCL;                     // bits 0 - 7, convert to int 
    batVolts = highADC<<8;                  // shift to bits 8 and 9
    batVolts = (batVolts+lowADC);           // full 10-bit number
                                            // LEAKY INTEGRATION
    batVolts = ((alpha * prevRead)+((10-alpha)*batVolts))/10;
    prevRead = batVolts;
            
 	return;
}
//disCharge ====================================================================

void disCharge()
{
    while(x>0)
        {
            b3High();                       // RED light ON
            b4Low();                        // GREEN light OFF
            checkStatus();                  // update
        }
    return;
}
//standBy ======================================================================

void standBy()
{
    while(x==0)
        { 
            b4High();                       // GREEN on
            b3Low();                        // RED off
            b0High();                       // LM117 constant voltage mode
            checkStatus();                  // update
        }
    return;
}
//checkStatus ==================================================================
void checkStatus()	
{
 static uint8_t y = 0;                      // previous status  
	
 x=(PINB & 0b10);                           // x=0 for charging, 1 discharging
	if (x==y)                               // no change
		{
			return;
		} 
	else
		{
			y = x;                          // debounce for 60ms
			_delay_ms(60);
			x=(PINB & 0b10);
			if(x == y)
				{
					return;
				}
		}
	return;					
}

#endif